""""

Test examples

"""


def test_register_asset():
    from examples.register_asset_squid import main
    main()


def test_register_upload_asset_remote():
    from examples.register_upload_asset_remote_agent import main
    main()
